﻿# -*- coding: utf_8 -*-
import struct, os, sys, stat, threading, math, time, array, fadeout

#The WAVE file format is a subset of Microsoft's RIFF specification for the storage of multimedia files. 
#http://ccrma.stanford.edu/CCRMA/Courses/422/projects/WaveFormat/

class waveFileLikeObj:
	def __init__(self, fileName, fileOffset, fileLength, fadeStartOffset = None, fadeSecs = 0, fadeMethod = 0):
		self.fileHeader = struct.pack('4si4s4sihhiihh4si', 'RIFF', 36+fileLength, 'WAVE', 'fmt ', 16, 1, 2, 44100, 176400, 4, 16, r'data', fileLength)
		self.fileHeaderLen	= len(self.fileHeader)
		#if platform.system() is "Windows": self.fileName = fileName.encode('mbcs')
		#else: self.fileName = fileName #currently unknow the encoding for SDL working under *nix
		self.fileName	= fileName
		if isinstance(fileName, unicode) is False:
			self.fileName	= fileName.decode(sys.getfilesystemencoding())
		self.fileOffset	= fileOffset
		self.fileLength	= fileLength
		self.inteOffset	= 0
		self.fadeStartOffset = fadeStartOffset
		self.fadeSecs	= fadeSecs
		self.fadeMethod = fadeMethod
	def seek(self,offset):
		self.inteOffset = offset
	def tell(self):
		return self.inteOffset
	def read(self, size = 0):
		try:
			tempOffset = self.inteOffset
			if self.inteOffset is -1:#EOF
				return str()
			tempArray = array.array('c')
			if size == 0:
				fp = file(self.fileName, "rb")
				if self.inteOffset < self.fileHeaderLen:
					fp.seek(self.fileOffset, 0)
					tempArray.extend(self.fileHeader[self.inteOffset:] + fp.read(self.fileLength))
				else:
					fp.seek(self.fileOffset+(self.inteOffset-self.fileHeaderLen), 0)
					tempArray.extend((fp.read(self.fileLength-(self.inteOffset-self.fileHeaderLen))))
				fp.close()
				self.inteOffset = -1
			elif size > 0:
				if self.inteOffset+size < self.fileHeaderLen:
					tempArray.extend(self.fileHeader[self.inteOffset:self.inteOffset+size])
				else:
					fp = file(self.fileName, "rb")
					if self.inteOffset < self.fileHeaderLen:
						if self.inteOffset+size <= self.fileHeaderLen:
							tempArray.extend(self.fileHeader[self.inteOffset:])
						else:
							fp.seek(self.fileOffset, 0)
							tempArray.extend(self.fileHeader[self.inteOffset:] + fp.read(size-(self.fileHeaderLen-self.inteOffset)))
					else:
						dir(fp) # a scope hotfix for win9x platform
						fp.seek(self.fileOffset+(self.inteOffset-self.fileHeaderLen), 0)
						if self.inteOffset+size > self.fileHeaderLen+self.fileLength:
							size = self.fileLength-(self.inteOffset-self.fileHeaderLen)
						tempArray.extend(fp.read(size))
					fp.close()
				self.inteOffset += size
				if self.inteOffset >= self.fileHeaderLen+self.fileLength:
					self.inteOffset = -1
			if self.fadeStartOffset is not None and self.fadeStartOffset<=self.fileOffset and tempOffset>=self.fileHeaderLen:
				# fadeWav = array.array('h', tempArray.tostring())
				value1 = float(self.fileOffset-self.fadeStartOffset)/4.
				value2 = float(abs(self.fadeSecs)*44100.)
				# if self.fadeMethod is 0:	fadeCalc = lambda d: pow(0.001, (value1+d)/(value2))
				# elif self.fadeMethod is 1:	fadeCalc = lambda d: 1.-(value1+d)/(value2)
				# elif self.fadeMethod is 2:
					# def fadeCalc(d):
						# d = (value1+d)/(value2)
						# return (pow(0.001,d)+(1.-d))/2.
				# elif self.fadeMethod is 3:	fadeCalc = lambda d: (math.cos(math.pi*(value1+d)/(value2))+1.)/2.
				# for cur_k in xrange(0, (len(tempStr))/2, 2):
					# d = fadeCalc(cur_k>>1)
					# fadeWav[cur_k] = int(fadeWav[cur_k]*d)
					# fadeWav[cur_k+1] = int(fadeWav[cur_k+1]*d)
				# return fadeWav.tostring() 
				return fadeout.fadeout(tempArray.tostring(), self.fadeMethod, value1, value2)
			return tempArray.tostring()
		except:
			import traceback
			traceback.print_exc()

class atomCondition:
	def __init__(self):
		self.EvnNotify	= threading.Event()
		self.EvnAck	= threading.Event()
	def wait(self):
		self.EvnNotify.wait()
		self.EvnNotify.clear()
	def ack(self):
		self.EvnAck.set()
	def notify(self):
		self.EvnNotify.set()
		self.EvnAck.wait()
		self.EvnAck.clear()

def	secToStr(secs, arg):
	if arg is 3:
		minutes = secs/60
		hours = minutes/60
		return '%02u:%02u:%02u'%(hours, minutes%60, secs%60)
	elif arg is 2:
		minutes = secs/60
		return '%02u:%02u'%(minutes, secs%60)
	else:
		return str(secs)

def locCertSizeFileInDir(dirpath,size):
	if dirpath is None: return None
	if os.path.exists(dirpath) is False: return None
	for name in os.listdir(dirpath):
		pathname = os.path.join(dirpath, name)
		if os.stat(pathname)[stat.ST_SIZE] == size:
			return pathname
	return None
